package cn.com.infosec.crypto.agreement.kdf;

import cn.com.infosec.asn1.ASN1EncodableVector;
import cn.com.infosec.asn1.DERObjectIdentifier;
import cn.com.infosec.asn1.DEROctetString;
import cn.com.infosec.asn1.DERSequence;
import cn.com.infosec.asn1.DERTaggedObject;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.DerivationFunction;
import cn.com.infosec.crypto.DerivationParameters;
import cn.com.infosec.crypto.Digest;

public class DHKEKGenerator implements DerivationFunction {
   private final Digest digest;
   private DERObjectIdentifier algorithm;
   private int keySize;
   private byte[] z;
   private byte[] partyAInfo;

   public DHKEKGenerator(Digest digest) {
      this.digest = digest;
   }

   private byte[] integerToBytes(int keySize) {
      byte[] var2;
      byte[] var10000 = var2 = new byte[4];
      var2[0] = (byte)(keySize >> 24);
      var2[1] = (byte)(keySize >> 16);
      var2[2] = (byte)(keySize >> 8);
      var10000[3] = (byte)keySize;
      return var10000;
   }

   public void init(DerivationParameters param) {
      DHKEKGenerator var10000 = this;
      DHKEKGenerator var10001 = this;
      DHKEKGenerator var10002 = this;
      DHKDFParameters var2;
      this.algorithm = (var2 = (DHKDFParameters)param).getAlgorithm();
      var10002.keySize = var2.getKeySize();
      var10001.z = var2.getZ();
      var10000.partyAInfo = var2.getExtraInfo();
   }

   public Digest getDigest() {
      return this.digest;
   }

   public int generateBytes(byte[] out, int outOff, int len) {
      if (out.length - len >= outOff) {
         long var4;
         long var10000 = var4 = (long)len;
         int var6 = this.digest.getDigestSize();
         if (var10000 <= 8589934591L) {
            int var13 = (int)((var4 + (var4 = (long)var6) - 1L) / var4);
            byte[] var5 = new byte[this.digest.getDigestSize()];
            int var7 = 1;

            for(int var8 = 0; var8 < var13; ++var8) {
               byte[] var10002 = this.z;
               this.digest.update(var10002, 0, var10002.length);
               ASN1EncodableVector var9;
               ASN1EncodableVector var10001 = var9 = new ASN1EncodableVector;
               var10001.<init>();
               ASN1EncodableVector var10;
               ASN1EncodableVector var18 = var10 = new ASN1EncodableVector;
               var10.<init>();
               var18.add(this.algorithm);
               var18.add(new DEROctetString(this.integerToBytes(var7)));
               var10001.add(new DERSequence(var10));
               byte[] var15;
               if ((var15 = this.partyAInfo) != null) {
                  DERTaggedObject var11;
                  DERTaggedObject var17 = var11 = new DERTaggedObject;
                  DEROctetString var12;
                  var12 = new DEROctetString.<init>(var15);
                  var17.<init>(true, 0, var12);
                  var9.add(var11);
               }

               DEROctetString var16;
               var16 = new DEROctetString.<init>(this.integerToBytes(this.keySize));
               var9.add(new DERTaggedObject(true, 2, var16));
               byte[] var14 = (new DERSequence(var9)).getDEREncoded();
               this.digest.update(var14, 0, var14.length);
               this.digest.doFinal(var5, 0);
               if (len > var6) {
                  System.arraycopy(var5, 0, out, outOff, var6);
                  outOff += var6;
                  len -= var6;
               } else {
                  System.arraycopy(var5, 0, out, outOff, len);
               }

               ++var7;
            }

            this.digest.reset();
            return len;
         } else {
            throw new IllegalArgumentException("Output length too large");
         }
      } else {
         throw new DataLengthException("output buffer too small");
      }
   }
}
